""" Utility functions and classes for SRP

Context : SRP
Module  : PM
Version : 1.1.1
Author  : Stefano Covino
Date    : 20/08/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (28/08/2013) First version.
        : (03/12/2013) Better naming.
        : (07/01/2014) Better organization of parameters.
        : (20/08/2014) Porting to Pilar3.
"""

from .Extra import Extra
from SRP.SRPMath.FastAngularDistance import FastAngularDistance



def minExtra (xxx_todo_changeme,taz,talt,oaz,oalt):
    (e_AAN,e_ZAN,e_AAE,e_ZAE,e_NPAE,e_BNP,e_AES,e_AEC,e_ZES,e_ZEC,e_AOFF,e_ZOFF,e_AS2A,e_AC2A,e_ZS2A,e_ZC2A,e_C5) = xxx_todo_changeme
    caz, calt = Extra((taz,talt),(e_AAN,e_ZAN,e_AAE,e_ZAE,e_NPAE,e_BNP,e_AES,e_AEC,e_ZES,e_ZEC,e_AOFF,e_ZOFF,e_AS2A,e_AC2A,e_ZS2A,e_ZC2A,e_C5))
    return FastAngularDistance(oaz,oalt,caz,calt).sum()




""" Utility functions and classes for SRP

Context : SRP
Module  : PM
Version : 1.3.0
Author  : Stefano Covino
Date    : 20/08/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (22/08/2012) First version.
        : (01/07/2013) Check of parameter names.
        : (22/08/2013) Just offsets.
        : (03/12/2013) Better naming of parameters.
        : (07/01/2014) Better organization of parameters.
        : (20/08/2014) Porting to Pilar3.
"""

import numpy

def cotg (ang):
    return 1./numpy.tan(ang)


def Full (xxx_todo_changeme, xxx_todo_changeme1):
    # AAN,ZAN: error in the leveling of the telescope toward north
    # AAE,ZAE: error in the leveling of the telescope toward east
    # NPAE: non-perpendicularity of the azimuth and elevation axis
    # BNP: non-perpendicularity of the optical and the elevation axis
    # AES, AEC, ZES, ZEC:  eccentricity of the encoders
    # AOFF: azimuth zero point correction
    # ZOFF: zenith distance zero point correction
    (az,alt) = xxx_todo_changeme
    (f_AAN,f_ZAN,f_AAE,f_ZAE,f_NPAE,f_BNP,f_AES,f_AEC,f_ZES,f_ZEC,f_AOFF,f_ZOFF) = xxx_todo_changeme1
    zd = 90.0 - alt
    #
    azr = numpy.radians(az)
    zdr = numpy.radians(zd)
    #
    naz = f_AAN * numpy.sin(azr) * cotg(zdr) - f_AAE * numpy.cos(azr) * cotg(zdr) + f_NPAE * cotg(zdr)  - f_BNP / numpy.sin(zdr) + f_AOFF + f_AES * numpy.sin(azr) + f_AEC * numpy.cos(azr)
    nzd = f_ZAN * numpy.cos(azr) + f_ZAE * numpy.sin(azr) + f_ZOFF + f_ZES * numpy.sin(zdr) + f_ZEC * numpy.cos(zdr)
    return naz, -nzd




""" Utility functions and classes for SRP

Context : SRP
Module  : PM
Version : 1.1.1
Author  : Stefano Covino
Date    : 20/08/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (23/08/2012) First version.
        : (22/08/2013) Improvement.
        : (20/08/2014) Porting to Pilar3.
"""


from .Simple import Simple
from SRP.SRPMath.FastAngularDistance import FastAngularDistance


def minSimple (xxx_todo_changeme,taz,talt,oaz,oalt):
    (s_AOFF,s_ZOFF) = xxx_todo_changeme
    caz, calt = Simple((taz,talt),(s_AOFF,s_ZOFF))
    return FastAngularDistance(oaz,oalt,caz,calt).sum()



""" Utility functions and classes for SRP

Context : SRP
Module  : PM
Version : 1.2.0
Author  : Stefano Covino
Date    : 20/08/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (22/08/2012) First version.
        : (22/08/2013) Just offsets.
        : (20/08/2014) Porting to Pilar3.
"""

import numpy


def cotg (ang):
    return 1./numpy.tan(ang)


def Classic (xxx_todo_changeme, xxx_todo_changeme1):
    # AN: error in the leveling of the telescope toward north
    # AE: error in the leveling of the telescope toward east
    # NPAE: non-perpendicularity of the azimuth and elevation axis
    # BNP: non-perpendicularity of the optical and the elevation axis
    # TF: sagging of the tube
    # AOFF: azimuth zero point correction
    # ZOFF: zenith distance zero point correction
    (az,alt) = xxx_todo_changeme
    (c_AN,c_AE,c_NPAE,c_BNP,c_TF,c_AOFF,c_ZOFF) = xxx_todo_changeme1
    zd = 90.0 - alt
    #
    azr = numpy.radians(az)
    zdr = numpy.radians(zd)
    #
    naz = -c_AN * numpy.sin(azr) * cotg(zdr) + c_AE * numpy.cos(azr) * cotg(zdr) + c_NPAE * cotg(zdr) - c_BNP / numpy.sin(zdr) + c_AOFF
    nzd = c_AN * numpy.cos(azr) + c_AE * numpy.sin(azr) + c_TF * numpy.sin(zdr) + c_ZOFF
    return naz, -nzd




""" Utility functions and classes for SRP

Context : SRP
Module  : PM
Version : 1.2.0
Author  : Stefano Covino
Date    : 20/08/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (28/08/2012) First version.
        : (03/12/2013) Better naming.
        : (07/01/2014) Better organization of parameters.
        : (20/08/2014) Porting to Pilar3.
"""

import numpy
from .Full import Full


def cotg (ang):
    return 1./numpy.tan(ang)


def Extra (xxx_todo_changeme, xxx_todo_changeme1):
    # AAN, ZAN: error in the leveling of the telescope toward north
    # AAE, ZAE: error in the leveling of the telescope toward east
    # NPAE: non-perpendicularity of the azimuth and elevation axis
    # BNP: non-perpendicularity of the optical and the elevation axis
    # AES, AEC, ZES, ZEC:  eccentricity of the encoders
    # AOFF: azimuth zero point correction
    # ZOFF: altitude zero point correction
    # ES2A = C1: Irregularities in the bearing race
    # EC2A = C2: Irregularities in the bearing race
    # C5: Empirical term
    (az,alt) = xxx_todo_changeme
    (e_AAN,e_ZAN,e_AAE,e_ZAE,e_NPAE,e_BNP,e_AES,e_AEC,e_ZES,e_ZEC,e_AOFF,e_ZOFF,e_AS2A,e_AC2A,e_ZS2A,e_ZC2A,e_C5) = xxx_todo_changeme1
    zd = 90.0 - alt
    #
    azr = numpy.radians(az)
    zdr = numpy.radians(zd)
    #
    naz, nalt = Full((az,alt),(e_AAN,e_ZAN,e_AAE,e_ZAE,e_NPAE,e_BNP,e_AES,e_AEC,e_ZES,e_ZEC,e_AOFF,e_ZOFF))
    naz = naz + (e_AS2A * numpy.cos(2*azr) + e_AC2A * numpy.cos(2*azr)) * cotg(zdr)
    nzd = -nalt
    nzd = nzd + e_ZS2A * numpy.sin(2*azr) + e_ZC2A * numpy.cos(2*azr) + e_C5 / numpy.sin(zdr)
    return naz, -nzd




""" Utility functions and classes for SRP

Context : SRP
Module  : PM
Version : 1.2.0
Author  : Stefano Covino
Date    : 20/08/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (22/08/2012) First version.
        : (22/08/2013) Just offsets.
        : (20/08/2014) Porting to Pilar3.
"""



def Simple (xxx_todo_changeme, xxx_todo_changeme1):
    # AOFF: azimuth zero point correction
    # ZOFF: zenith distance zero point correction
    (az,alt) = xxx_todo_changeme
    (s_AOFF,s_ZOFF) = xxx_todo_changeme1
    zd = 90.0 - alt
    #
    naz = s_AOFF+az-az
    nzd = s_ZOFF+zd-zd
    return naz, -nzd

